@@ -2,11 +2,11 @@ from __future__ import annotations
 
 import contextlib
 import os
-import urllib
 import warnings
-from collections import Counter
+from collections import Counter, defaultdict
 from fnmatch import fnmatch
 from importlib import metadata
+from logging import getLogger
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
@@ -25,6 +25,7 @@ from typing import (
     Tuple,
     Union,
 )
+from urllib import parse
 
 from psygnal import Signal, SignalGroup
 
@@ -37,6 +38,7 @@ from .types import PathLike, PythonName
 if TYPE_CHECKING:
     from .manifest.contributions import (
         CommandContribution,
+        MenuCommand,
         MenuItem,
         ReaderContribution,
         SampleDataContribution,
@@ -52,6 +54,8 @@ if TYPE_CHECKING:
     InclusionSet = Union[AbstractSetIntStr, MappingIntStrAny, None]
     DisposeFunction = Callable[[], None]
 
+logger = getLogger(__name__)
+
 __all__ = ["PluginContext", "PluginManager"]
 PluginName = str  # this is `PluginManifest.name`
 
@@ -140,7 +144,7 @@ class _ContributionsIndex:
             yield from (r for pattern, r in self._readers if pattern == "")
         else:
             # ensure not a URI
-            if not urllib.parse.urlparse(path).scheme:
+            if not parse.urlparse(path).scheme:
                 # lower case the extension for checking manifest pattern
                 base = os.path.splitext(Path(path).stem)[0]
                 ext = "".join(Path(path).suffixes)
@@ -232,6 +236,17 @@ class PluginManager:
         self._manifests: Dict[PluginName, PluginManifest] = {}
         self.events = PluginManagerEvents(self)
         self._npe1_adapters: List[NPE1Adapter] = []
+        self._command_menu_map: Dict[str, Dict[str, Dict[str, List[MenuCommand]]]] = (
+            # for each manifest, maps command IDs to menu IDs to list of MenuCommands
+            # belonging to that menu
+            # i.e. manifest -> command -> menu_id -> list[MenuCommand]
+            # we keep a list of MenuCommands even though we've already indexed by
+            # command ID **and** menu ID because it's possible to declare the
+            # same command in the same menu with different `when` and `group`
+            # fields. This seems an unlikely use case, but it is possible in
+            # the current schema.
+            defaultdict(lambda: defaultdict(lambda: defaultdict(list)))
+        )
 
         # up to napari 0.4.15, discovery happened in the init here
         # so if we're running on an older version of napari, we need to discover
@@ -241,11 +256,8 @@ class PluginManager:
             pass
         else:  # pragma: no cover
             vsplit = nv.split(".")[:4]
-            if (
-                "dev" in nv
-                and vsplit < ["0", "4", "16", "dev4"]
-                or "dev" not in nv
-                and vsplit < ["0", "4", "16"]
+            if ("dev" in nv and vsplit < ["0", "4", "16", "dev4"]) or (
+                "dev" not in nv and vsplit < ["0", "4", "16"]
             ):
                 self.discover()
 
@@ -355,14 +367,27 @@ class PluginManager:
             self._npe1_adapters.append(manifest)
         else:
             self._contrib.index_contributions(manifest)
+            self._populate_command_menu_map(manifest)
         self.events.plugins_registered.emit({manifest})
 
+    def _populate_command_menu_map(self, manifest: PluginManifest):
+        # map of manifest -> command -> menu_id -> list[items]
+        menu_map = self._command_menu_map[manifest.name]  # just for conciseness below
+        for menu_id, menu_items in manifest.contributions.menus.items() or ():
+            # command IDs are keys in map
+            # each value is a dict menu_id: list of MenuCommands
+            # for the command and menu
+            for item in menu_items:
+                if (command_id := getattr(item, "command", None)) is not None:
+                    menu_map[command_id][menu_id].append(item)
+
     def unregister(self, key: PluginName):
         """Unregister plugin named `key`."""
         if key not in self._manifests:
             raise ValueError(f"No registered plugin named {key!r}")  # pragma: no cover
         self.deactivate(key)
         self._contrib.remove_contributions(key)
+        self._command_menu_map.pop(key)
         self._manifests.pop(key)
 
     def activate(self, key: PluginName) -> PluginContext:
@@ -445,6 +470,7 @@ class PluginManager:
         mf = self._manifests.get(plugin_name)
         if mf is not None:
             self._contrib.index_contributions(mf)
+            self._populate_command_menu_map(mf)
         self.events.enablement_changed({plugin_name}, {})
 
     def disable(self, plugin_name: PluginName) -> None:
@@ -464,6 +490,7 @@ class PluginManager:
 
         self._disabled_plugins.add(plugin_name)
         self._contrib.remove_contributions(plugin_name)
+        self._command_menu_map.pop(plugin_name, None)
         self.events.enablement_changed({}, {plugin_name})
 
     def is_disabled(self, plugin_name: str) -> bool:
@@ -675,12 +702,8 @@ class PluginManager:
 
         for writer in self.iter_compatible_writers(layer_types):
             if not plugin_name or writer.command.startswith(plugin_name):
-                if (
-                    ext
-                    and ext in writer.filename_extensions
-                    or not ext
-                    and len(layer_types) != 1
-                    and not writer.filename_extensions
+                if (ext and ext in writer.filename_extensions) or (
+                    not ext and len(layer_types) != 1 and not writer.filename_extensions
                 ):
                     return writer, path
                 elif not ext and len(layer_types) == 1:  # No extension, single layer.
@@ -692,6 +715,10 @@ class PluginManager:
         # Nothing got found
         return None, path
 
+    def get_shimmed_plugins(self) -> List[str]:
+        """Return a list of all shimmed plugin names."""
+        return [mf.name for mf in self.iter_manifests() if mf.npe1_shim]
+
 
 class PluginContext:
     """An object that can contain information for a plugin over its lifetime."""
@@ -710,7 +737,10 @@ class PluginContext:
 
     def _dispose(self):
         while self._disposables:
-            self._disposables.pop()()
+            try:
+                self._disposables.pop()()
+            except Exception as e:
+                logger.warning(f"Error while disposing {self.plugin_key}; {e}")
 
     def register_command(self, id: str, command: Optional[Callable] = None):
         """Associate a callable with a command id."""
