@@ -1,4 +1,4 @@
 from ._package_metadata import PackageMetadata
 from .schema import ENTRY_POINT, PluginManifest
 
-__all__ = ["PackageMetadata", "PluginManifest", "ENTRY_POINT"]
+__all__ = ["ENTRY_POINT", "PackageMetadata", "PluginManifest"]
